require 'jwt'
require 'httparty'
require 'time'

module AppleMusic

  # THANKS https://medium.com/@defv/connecting-to-the-apple-music-api-from-a-ruby-on-rails-application-eba9d12f5cf9
  class TokenFactory

    def self.generate(secret_key_path:, team_id:, key_id:)
      ecdsa_key = OpenSSL::PKey::EC.new File.read(secret_key_path)
      ecdsa_public = OpenSSL::PKey::EC.new ecdsa_key
      ecdsa_public.private_key = nil
      JWT.encode TokenFactory.authentication_payload(team_id), ecdsa_public, 'ES256', { kid: key_id }
    end

    private

    def self.authentication_payload(team_id)
      {
        iss: team_id,
        iat: Time.now.to_i - 100,
        exp: Time.now.to_i + 12000
      }
    end

  end

  class Client

    def initialize(token)
      @token = token
    end

    def get(resource)
      response = HTTParty.get(
        "https://api.music.apple.com#{resource}",
        {headers: headers})

      if response.code == 200
        JSON.parse(response.body)
      else
        puts "NON 200 CODE for resource #{resource}"
        puts response
      end
    end

    def post(resource)
      HTTParty.post(
        "https://api.music.apple.com#{resource}",
        {headers: headers}
      )
    end

    def headers
      {
        'Authorization': "Bearer #{@token}"
      }
    end

  end

  class PaginatedResponse
    include Enumerable

    def initialize(client, response)
      @client = client
      @response = response
    end

    def each(&block)
      return to_enum(:each) unless block_given?
      return if @response.nil? or @response["data"].nil?

      @response["data"].each(&block)
      return if @response["next"].nil?
      PaginatedResponse.new(@client, @client.get(@response["next"])).each(&block)
    end

  end


  class DataType < Struct
    def initialize(args)
      args.merge(args['attributes'])
          .filter { |k,_| members.include?(k.to_sym) }
          .each { |k,v| public_send("#{k}=", v) }
      freeze
    end
  end

end